--AH:ScriptId=6b8bd3be-6698-4c45-9d18-13435ad3bd65;2961
DROP TABLE [Privileges]
GO
DROP TABLE [RoleTasks]
GO
DROP TABLE [Tasks]
GO
DROP TABLE [Roles]
GO
DROP TABLE [UserGroups]
GO
DROP TABLE [Users]
GO
DROP TABLE [Groups]
GO

CREATE TABLE [Roles]
(
	[Role_Id] INT IDENTITY(1,1) NOT NULL
		CONSTRAINT [PK__Roles]
		PRIMARY KEY NONCLUSTERED ([Role_Id]),
	[Role_Name] NVARCHAR(50) NOT NULL,
	CONSTRAINT [UQ__Roles]
		UNIQUE NONCLUSTERED ([Role_Name])
)

CREATE TABLE [Tasks]
(
	[Task_Id] INT NOT NULL
		CONSTRAINT [PK__Tasks]
		PRIMARY KEY CLUSTERED ([Task_Id]),
	[Task_Name] VARCHAR(50) NOT NULL,
    [EnvironmentScopable_Indicator] YNINDICATOR NOT NULL,
	CONSTRAINT [UQ__Tasks]
		UNIQUE NONCLUSTERED ([Task_Name])
)

GO

CREATE TABLE [RoleTasks]
(
	[Role_Id] INT NOT NULL,
		CONSTRAINT [FK__RoleTasks__Roles]
		FOREIGN KEY ([Role_Id]) REFERENCES [Roles] ([Role_Id]) ON DELETE CASCADE,
	[Task_Id] INT NOT NULL,
		CONSTRAINT [FK__RoleTasks__Tasks]
		FOREIGN KEY ([Task_Id]) REFERENCES [Tasks] ([Task_Id]) ON DELETE CASCADE,
	CONSTRAINT [PK__RoleTasks]
		PRIMARY KEY ([Role_Id], [Task_Id])
)
GO

CREATE TABLE [Privileges](
	[Privilege_Id] INT IDENTITY(1,1) NOT NULL
		CONSTRAINT [PK__Privileges] 
		PRIMARY KEY NONCLUSTERED ([Privilege_Id]),
	[Principal_Name] [USERNAME] NOT NULL,
    [PrivilegeType_Code] CHAR(1) NOT NULL,
	[PrincipalType_Code] CHAR(1) NOT NULL,
	[Role_Id] INT NOT NULL,
    [Environment_Id] INT NULL,
    [UserDirectory_Id] INT NOT NULL,

	CONSTRAINT [UQ__Privileges] 
		UNIQUE NONCLUSTERED ([Principal_Name],[PrincipalType_Code],[Role_Id]),
	CONSTRAINT [CK_Privileges__PrincipalType_Code_Domain] CHECK  (([PrincipalType_Code]='G' OR [PrincipalType_Code]='U')),
    CONSTRAINT [FK__Privileges__Environments] FOREIGN KEY ([Environment_Id])
               REFERENCES [Environments] ([Environment_Id])
               ON DELETE CASCADE,

    CONSTRAINT [FK__Privileges__UserDirectories]
		FOREIGN KEY ([UserDirectory_Id])
		REFERENCES [UserDirectories] ([UserDirectory_Id])
)

GO

CREATE TABLE [Users] (
	[User_Name] USERNAME NOT NULL
		CONSTRAINT [PK__Users] 
		PRIMARY KEY,
	[Display_Name] NVARCHAR(100) NOT NULL,
	[Email_Address] VARCHAR(250) NULL,
	[Salt_Bytes] BINARY(10) NULL,
	[Password_Bytes] BINARY(20) NULL,
    [ChangePasswordOnLogin_Indicator] YNINDICATOR NOT NULL
)
GO

CREATE TABLE [Groups](
	[Group_Name] NVARCHAR(50) NOT NULL,
    CONSTRAINT [PK__Groups] PRIMARY KEY ([Group_Name])
)
GO

CREATE TABLE [UserGroups](
	[User_Name] USERNAME NOT NULL,
	[Group_Name] NVARCHAR(50) NOT NULL,
   CONSTRAINT [PK__UserGroups] PRIMARY KEY ([User_Name],[Group_Name]),
   CONSTRAINT [FK__UserGroups__Groups] FOREIGN KEY([Group_Name]) REFERENCES [Groups] ([Group_Name]),
   CONSTRAINT [FK__UserGroups__Users] FOREIGN KEY([User_Name]) REFERENCES [Users] ([User_Name])
)
GO

INSERT [Tasks] ([Task_Id], [Task_Name], [EnvironmentScopable_Indicator])
SELECT 1, 'Admin_ConfigureOtter', 'N' UNION ALL
SELECT 2, 'Admin_ViewErrorLog', 'N' UNION ALL
SELECT 3, 'Assets_Manage', 'N' UNION ALL
SELECT 4, 'Assets_View', 'N' UNION ALL
SELECT 5, 'ConfigurationPlans_Manage', 'Y' UNION ALL
SELECT 6, 'ConfigurationPlans_View', 'Y' UNION ALL
SELECT 7, 'Credentials_Manage', 'Y' UNION ALL
SELECT 8, 'Credentials_ViewPasswords', 'Y' UNION ALL
SELECT 9, 'Environments_Manage', 'N' UNION ALL
SELECT 10, 'Executions_View', 'Y' UNION ALL
SELECT 11, 'Executions_ViewDebugLogs', 'Y' UNION ALL
SELECT 12, 'Jobs_Manage', 'N' UNION ALL
SELECT 13, 'Jobs_ViewHistory', 'Y' UNION ALL
SELECT 14, 'OrchestrationPlans_Manage', 'N' UNION ALL
SELECT 15, 'OrchestrationPlans_View', 'N' UNION ALL
SELECT 16, 'Roles_Manage', 'N' UNION ALL
SELECT 17, 'Servers_Manage', 'N' UNION ALL
SELECT 18, 'Servers_View', 'Y' UNION ALL
SELECT 19, 'Variables_Manage', 'Y' UNION ALL
SELECT 20, 'Variables_View', 'Y'

GO


INSERT INTO [Roles] ([Role_Name])
 SELECT 'Administrator' UNION ALL
 SELECT 'Orchestrator' UNION ALL
 SELECT 'Configurer' UNION ALL
 SELECT 'Viewer'

GO

INSERT [RoleTasks] ([Role_Id], [Task_Id])

       SELECT R.[Role_Id], T.[Task_Id] 
         FROM [Tasks] T
   INNER JOIN [Roles] R 
           ON R.[Role_Name] = 'Administrator'

 UNION ALL 

       SELECT R.[Role_Id], T.[Task_Id] 
         FROM [Tasks] T
   INNER JOIN [Roles] R 
           ON R.[Role_Name] = 'Orchestrator'
          AND T.[Task_Id] <> 1

 UNION ALL 

       SELECT R.[Role_Id], T.[Task_Id] 
         FROM [Tasks] T
   INNER JOIN [Roles] R 
           ON R.[Role_Name] = 'Configurer'
          AND T.[Task_Id] IN (2,3,4,5,6,10,13,18,19,20)

 UNION ALL 

       SELECT R.[Role_Id], T.[Task_Id] 
         FROM [Tasks] T
   INNER JOIN [Roles] R 
           ON R.[Role_Name] = 'Viewer'
           AND T.[Task_Id] IN (4,6,13,18,20)
GO

INSERT INTO [Users] ([User_Name], [Display_Name], [Email_Address], [Salt_Bytes], [Password_Bytes], [ChangePasswordOnLogin_Indicator])
  VALUES ('Admin', 'Administrator', NULL, 0xE1A8CF26199D1E12D4C5, 0x2ACD2669D6B77ED74DA50E1E4BD5A6AA3744F13E, 'N')
GO

INSERT [Groups]
  SELECT 'Administrators' UNION ALL
  SELECT 'Orchestrators' UNION ALL
  SELECT 'Configurers' UNION ALL
  SELECT 'Viewers'
GO

INSERT [UserGroups] ([Group_Name], [User_Name])
  VALUES ('Administrators', 'Admin')
GO

INSERT [Privileges] ([Principal_Name], [PrivilegeType_Code], [PrincipalType_Code], [Role_Id], [Environment_Id], [UserDirectory_Id])
 SELECT 'Administrators', 'G', 'G', [Role_Id], NULL, 1
   FROM [Roles] WHERE [Role_Name] = 'Administrator'
  UNION ALL 
 SELECT 'Orchestrators', 'G', 'G', [Role_Id], NULL, 1
   FROM [Roles] WHERE [Role_Name] = 'Orchestrator'
  UNION ALL 
 SELECT 'Configurers', 'G', 'G', [Role_Id], NULL, 1
   FROM [Roles] WHERE [Role_Name] = 'Configurer'
  UNION ALL 
 SELECT 'Viewers', 'G', 'G', [Role_Id], NULL, 1
   FROM [Roles] WHERE [Role_Name] = 'Viewer'
GO